/*
	PlayCmd.cc

	Will Eighteen, year?... lost in the mists...: will@opaline.demon.co.uk
*/

#include "wave/waveclass.h"
#include "AppWindow.h"
#include "PlayCmd.h"
#include "OpResources.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <MessageDialogManager.h>
#include <FileAccess.h>


PlayCmd::PlayCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_playCmd[0] = 0;
	_currentWave[0] = 0;
	_numWaves = 0;
	_numChannels = 0;
	_sampleRate = 0;
	_bitsPerSample = 0;
	_workingDir[0] = 0;
	_startSel = 0;
	_endSel = 0;
}


void PlayCmd::doCommand()
{
	char buf[1280];
	char bufCopy[1280];
	char reportFile[512];

	if ( wavePak->Count() == 0 )
	{
		if ( mixwaveResources->enableMessagePopups() == True )
		{
			strcpy ( buf, "Play: no waves loaded." );
			messageDialogManager->post ( buf );
		}

		return;
	}
	else
		_numWaves = wavePak->Count();

	if ( strlen ( mixwaveResources->currentWave() ) == 0 )	// there isn't one
	{
		if ( mixwaveResources->enableMessagePopups() == True )
		{
			strcpy ( buf, "Play: no wave selected; use\n'Options/Wave control'." );
			messageDialogManager->post ( buf );
		}
	}

	strncpy ( _playCmd,  mixwaveResources->playCmd(), sizeof ( _playCmd ) );
	_playCmd [ sizeof ( _playCmd ) - 1 ] = 0;
	strncpy ( _currentWave, mixwaveResources->currentWave(), sizeof ( _currentWave ) );
	_currentWave [ sizeof ( _currentWave ) - 1 ] = 0;

	for ( unsigned int count = 0; count < _numWaves; count++ )
	{
		if ( strcmp ( (*wavePak)[count].pWave->name(), _currentWave ) == 0 )
		{
			_waveNum = count + 1;
			count = _numWaves;
		}
	}
	
	if ( (*wavePak)[_waveNum-1].fLock == ACTIVE )
		return;

	(*wavePak)[_waveNum-1].fLock = ACTIVE;
	_numChannels = (*wavePak)[_waveNum-1].pWave->numChannels();
	_sampleRate = (*wavePak)[_waveNum-1].pWave->sampleRate();
	_bitsPerSample = (*wavePak)[_waveNum-1].pWave->bitsPerSample();
	strncpy ( _workingDir, mixwaveResources->workingDir(), sizeof ( _workingDir ) );
	_workingDir [ sizeof ( _workingDir ) - 1 ] = 0;
	_startSel = (*wavePak)[_waveNum-1].startSel;
	_endSel = (*wavePak)[_waveNum-1].endSel;

	strncpy ( buf, _playCmd, sizeof ( _playCmd ) );
	_playCmd [ sizeof ( _playCmd ) - 1 ] = 0;
	strncat ( buf, mixwaveResources->playPreamble(), sizeof ( buf ) - strlen ( buf ) - 1);
	strncpy ( reportFile, _workingDir, sizeof ( reportFile ) );
	reportFile [ sizeof ( reportFile ) - 1 ] = 0;
	strncat ( reportFile, "mixwavePlay.out", sizeof ( reportFile ) - strlen ( reportFile ) - 1 );

	if ( (*wavePak)[_waveNum-1].opMode == 0 )	// use entire wave
		strncat ( buf, _currentWave, sizeof ( buf ) - strlen ( buf ) - 1 );
	else
	{
		long numSamples = _endSel - _startSel;	
		long blockLength = mixwaveResources->blockLength();
		long endPos = _endSel<<( ( _bitsPerSample / 8 ) - 1 );
		char tmpFile[1024];
		Wave *pWave = new Wave;
		
		pWave->FillHeader ( _numChannels, _sampleRate, _bitsPerSample, numSamples );
		strncpy ( tmpFile, _workingDir, sizeof ( tmpFile ) - strlen ( tmpFile ) - 1 );
		strncat ( tmpFile, "mixwave-temp.wav", sizeof ( tmpFile ) - strlen ( tmpFile ) - 1 );

		if ( pWave->SaveHeader ( tmpFile ) == False )
		{
			snprintf ( buf, sizeof ( buf ), "Error  saving %s", tmpFile );
			buf [ sizeof ( buf ) - 1 ] = 0;
			messageDialogManager->post ( buf );
			delete pWave;
			return;
		}

		(*wavePak)[_waveNum-1].pWave->filePos = _startSel<<( ( _bitsPerSample / 8 ) - 1 );

		while ( (*wavePak)[_waveNum-1].pWave->filePos < _endSel<<( ( _bitsPerSample / 8 ) - 1 ) )
		{
			if ( blockLength > ( endPos - (*wavePak)[_waveNum-1].pWave->filePos ) )
				blockLength = endPos - (*wavePak)[_waveNum-1].pWave->filePos;

			(*wavePak)[_waveNum-1].pWave->LoadBlock ( (*wavePak)[_waveNum-1].pWave->filePos, blockLength );		
			(*wavePak)[_waveNum-1].pWave->AppendBlock ( tmpFile, _numChannels, _sampleRate, _bitsPerSample, blockLength>>1 );
			(*wavePak)[_waveNum-1].pWave->filePos += blockLength;
		}

		strncat ( buf, tmpFile, sizeof ( buf ) - strlen ( buf ) - 1 );		
		delete pWave;
	}

	strncat ( buf, mixwaveResources->playPostamble(), sizeof ( buf ) - strlen ( buf ) - 1 );	// error to file
	strncat ( buf, reportFile, sizeof ( buf ) - strlen ( buf ) - 1 );
	strncat ( buf, mixwaveResources->playPostfix(), sizeof ( buf ) - strlen ( buf ) - 1 );	// error to file
	strncpy ( bufCopy, buf, sizeof ( bufCopy ) );
	bufCopy [ sizeof ( bufCopy ) - 1 ] = 0;

	int error = sysCall ( buf );

	// buf is stripped by sysCall...
	if ( error )
	{
		char msg [1024];
		snprintf ( msg, sizeof ( msg ) - 1, "PlayCmd: system call error in\n%s", bufCopy );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
		return;
	}

// while !file ? loop? (on expectation it gets written)
	sysCall ( "sleep 1s" );	// delay to give sys call time to write file

	FileAccess *in = new FileAccess ( reportFile, "r" );
	
	if ( in->errNum() == 0 )
	{
		strncat ( bufCopy, "\n", sizeof ( bufCopy ) - strlen ( bufCopy ) - 1 );
//		strncpy ( buf, in->readF(), sizeof ( buf ) );
in->readF ( buf, sizeof ( buf ) );
		buf [ sizeof ( buf ) - 1 ] = 0;
		strncat ( bufCopy, buf, sizeof ( bufCopy ) - strlen ( bufCopy ) - 1 );

		if ( strlen ( buf ) > 0 )	// error report in file
			messageDialogManager->post ( bufCopy );
	}
	else
		messageDialogManager->post ( in->error() );
	
	delete in;
	snprintf ( buf, sizeof ( buf ), "%s %s", mixwaveResources->removeCmd(), reportFile );
	buf [ sizeof ( buf ) - 1 ] = 0;
	sysCall ( buf );

	(*wavePak)[_waveNum-1].fLock = INACTIVE;
	// n.b. temp file is left in case anyone wants the selection...	
}


int PlayCmd::sysCall ( const char *command )
{	// code ripped from GNU 'system' man page
   int pid, status;

   if ( command == 0 )
       return 1;
	   
   pid = fork();

   if ( pid == -1 )
       return -1;
	   
   if ( pid == 0 )
   {
       char *argv[4];
       argv[0] = "sh";
       argv[1] = "-c";
       argv[2] = ( char * ) command;
       argv[3] = 0;
	   execve( "/bin/sh", argv, environ );
       exit ( 127 );
   }

   do
   {
		if ( waitpid ( pid, &status, 0 ) == -1 )
		{
           if ( errno != EINTR )
               return -1;
		}
		else
           return status;   
   } while ( 1 );
}
